@@ -54,51 +54,41 @@ Conversion of power of 2 sample-rates(Hz) to Monitor periods(ms)
 
 """
 
+import abc
 import numpy
-from tvb.datatypes.time_series import (TimeSeries, TimeSeriesRegion,
-    TimeSeriesEEG, TimeSeriesMEG, TimeSeriesSEEG, TimeSeriesSurface)
-from tvb.simulator.common import get_logger, simple_gen_astr
+from tvb.datatypes.time_series import (TimeSeries, TimeSeriesRegion, TimeSeriesEEG, TimeSeriesMEG, TimeSeriesSEEG,
+                                       TimeSeriesSurface)
+from tvb.simulator import noise
 import tvb.datatypes.sensors as sensors_module
-from tvb.datatypes.sensors import SensorsMEG, SensorsInternal, SensorsEEG, Sensors
-import tvb.datatypes.arrays as arrays
-from tvb.datatypes.cortex import Cortex
+from tvb.datatypes.sensors import SensorsEEG
 from tvb.datatypes.region_mapping import RegionMapping
-from tvb.datatypes.connectivity import Connectivity
-from tvb.datatypes.projections import (ProjectionMatrix,
-    ProjectionSurfaceEEG, ProjectionSurfaceMEG, ProjectionSurfaceSEEG)
+from tvb.datatypes.projections import (ProjectionMatrix, ProjectionSurfaceEEG, ProjectionSurfaceMEG,
+                                       ProjectionSurfaceSEEG)
 import tvb.datatypes.equations as equations
-import tvb.basic.traits.util as util
-import tvb.basic.traits.types_basic as basic
-import tvb.basic.traits.core as core
 from tvb.simulator.common import iround, numpy_add_at
+from tvb.basic.neotraits.api import HasTraits, Attr, NArray, Float, narray_describe
 
 
-LOG = get_logger(__name__)
 
-
-class Monitor(core.Type):
+class Monitor(HasTraits):
     """
     Abstract base class for monitor implementations.
-
     """
 
-    # list of class names not shown in UI
-    _base_classes = ['Monitor', 'Projection', 'ProgressLogger']
-
-    period = basic.Float(
-        label = "Sampling period (ms)", order=10,
-        default = 0.9765625, #ms. 0.9765625 => 1024Hz #ms, 0.5 => 2000Hz
-        doc = """Sampling period in milliseconds, must be an integral multiple
+    period = Float(
+        label="Sampling period (ms)",  # order = 10
+        default=0.9765625,  # ms. 0.9765625 => 1024Hz #ms, 0.5 => 2000Hz
+        doc="""Sampling period in milliseconds, must be an integral multiple
         of integration-step size. As a guide: 2048 Hz => 0.48828125 ms ;  
         1024 Hz => 0.9765625 ms ; 512 Hz => 1.953125 ms.""")
 
-    variables_of_interest = arrays.IntegerArray(
-        label = "Model variables to watch", order=11,
-        doc = ("Indices of model's variables of interest (VOI) that this monitor should record. "
-               "Note that the indices should start at zero, so that if a model offers VOIs V, W and "
-               "V+W, and W is selected, and this monitor should record W, then the correct index is 0.")
-        #order = -1
-    )
+    variables_of_interest = NArray(
+        dtype=int,
+        label="Model variables to watch",  # order=11,
+        doc=("Indices of model's variables of interest (VOI) that this monitor should record. "
+             "Note that the indices should start at zero, so that if a model offers VOIs V, W and "
+             "V+W, and W is selected, and this monitor should record W, then the correct index is 0."),
+        required=False)
 
     istep = None
     dt = None
@@ -134,21 +124,16 @@ class Monitor(core.Type):
         rather implement the `sample` method.
 
         """
-
         return self.sample(step, observed)
 
+    @abc.abstractmethod
     def sample(self, step, state):
         """
         This method provides monitor output, and should be overridden by subclasses.
 
         """
 
-        raise NotImplementedError(
-            "The Monitor base class does not provide any observation model and "
-            "should be subclasses with an implementation of the `sample` method.")
-
-    def create_time_series(self, storage_path, connectivity=None, surface=None,
-                           region_map=None, region_volume_map=None):
+    def create_time_series(self, connectivity=None, surface=None, region_map=None, region_volume_map=None):
         """
         Create a time series instance that will be populated by this monitor
         :param surface: if present a TimeSeriesSurface is returned
@@ -156,27 +141,18 @@ class Monitor(core.Type):
         Otherwise a plain TimeSeries will be returned
         """
         if surface is not None:
-            return TimeSeriesSurface(storage_path=storage_path,
-                                     surface=surface,
+            return TimeSeriesSurface(surface=surface.region_mapping_data.surface,
                                      sample_period=self.period,
-                                     title='Surface ' + self.__class__.__name__,
-                                     **self._transform_user_tags())
+                                     title='Surface ' + self.__class__.__name__)
         if connectivity is not None:
-            return TimeSeriesRegion(storage_path=storage_path,
-                                    connectivity=connectivity,
+            return TimeSeriesRegion(connectivity=connectivity,
                                     region_mapping=region_map,
                                     region_mapping_volume=region_volume_map,
                                     sample_period=self.period,
-                                    title='Regions ' + self.__class__.__name__,
-                                    **self._transform_user_tags())
+                                    title='Regions ' + self.__class__.__name__)
 
-        return TimeSeries(storage_path=storage_path,
-                          sample_period=self.period,
-                          title=' ' + self.__class__.__name__,
-                          **self._transform_user_tags())
-
-    def _transform_user_tags(self):
-        return {}
+        return TimeSeries(sample_period=self.period,
+                          title=' ' + self.__class__.__name__)
 
 
 class Raw(Monitor):
@@ -191,17 +167,18 @@ class Raw(Monitor):
     """
     _ui_name = "Raw recording"
 
-    period = basic.Float(
-        label = "Sampling period is ignored for Raw Monitor",
-        order = -1)
+    period = Float(default=0.0, label="Sampling period is ignored for Raw Monitor")
+    # order = -1
 
-    variables_of_interest = arrays.IntegerArray(
-        label = "Raw Monitor sees all!!! Resistance is futile...",
-        order = -1)
+    variables_of_interest = NArray(
+        dtype=int,
+        label="Raw Monitor sees all!!! Resistance is futile...",
+        required=False)
+    # order = -1
 
     def config_for_sim(self, simulator):
         if self.period != simulator.integrator.dt:
-            LOG.debug('Raw period not equal to integration time step, overriding')
+            self.log.debug('Raw period not equal to integration time step, overriding')
         self.period = simulator.integrator.dt
         super(Raw, self).config_for_sim(simulator)
         self.istep = 1
@@ -242,20 +219,23 @@ class SpatialAverage(Monitor):
     """
     _ui_name = "Spatial average with temporal sub-sample"
 
-    spatial_mask = arrays.IntegerArray( #TODO: Check it's a vector of length Nodes (like region mapping for surface)
-        label = "An index mask of nodes into areas",
-        doc = """A vector of length==nodes that assigns an index to each node
+    spatial_mask = NArray(  #TODO: Check it's a vector of length Nodes (like region mapping for surface)
+        dtype=int,
+        label="An index mask of nodes into areas",
+        required=False,
+        doc="""A vector of length==nodes that assigns an index to each node
             specifying the "region" to which it belongs. The default usage is
             for mapping a surface based simulation back to the regions used in 
             its `Long-range Connectivity.`""")
-    
-    default_mask = basic.Enumerate(
-                              label = "Default Mask",
-                              options = ["cortical", "hemispheres"],
-                              default = ["hemispheres"],
-                              doc = r"""Fallback in case spatial mask is none and no surface provided 
-                              to use either connectivity hemispheres or cortical attributes.""",
-                              order = -1)
+
+    default_mask = Attr(
+        str,
+        choices=("cortical", "hemispheres"),
+        default="hemispheres",
+        label="Default Mask",
+        doc=("Fallback in case spatial mask is none and no surface provided" 
+             "to use either connectivity hemispheres or cortical attributes."))
+        # order = -1)
 
     def config_for_sim(self, simulator):
 
@@ -264,7 +244,7 @@ class SpatialAverage(Monitor):
         self.is_default_special_mask = False
 
         # setup given spatial mask or default to region mapping
-        if self.spatial_mask.size == 0:
+        if self.spatial_mask is None:
             self.is_default_special_mask = True
             if not (simulator.surface is None):
                 self.spatial_mask = simulator.surface.region_mapping
@@ -273,14 +253,14 @@ class SpatialAverage(Monitor):
                 if self.default_mask[0] == 'cortical':
                     if conn is not None and conn.cortical is not None and conn.cortical.size > 0:
                         ## Use as spatial-mask cortical/non cortical areas
-                        self.spatial_mask = [int(c) for c in conn.cortical]
+                        self.spatial_mask = numpy.array([int(c) for c in conn.cortical])
                     else:
                         msg = "Must fill Spatial Mask parameter for non-surface simulations when using SpatioTemporal monitor!"
                         raise Exception(msg)
                 if self.default_mask[0] == 'hemispheres':
                     if conn is not None and conn.hemispheres is not None and conn.hemispheres.size > 0:
                         ## Use as spatial-mask left/right hemisphere
-                        self.spatial_mask = [int(h) for h in conn.hemispheres]
+                        self.spatial_mask = numpy.array([int(h) for h in conn.hemispheres])
                     else:
                         msg = "Must fill Spatial Mask parameter for non-surface simulations when using SpatioTemporal monitor!"
                         raise Exception(msg)
@@ -297,14 +277,17 @@ class SpatialAverage(Monitor):
                     "contiguous set of indices starting from zero.")
             raise Exception(msg)
 
-        util.log_debug_array(LOG, self.spatial_mask, "spatial_mask", owner=self.__class__.__name__)
+        self.log.debug("spatial_mask")
+        self.log.debug(narray_describe(self.spatial_mask))
         spatial_sum = numpy.zeros((number_of_nodes, number_of_areas))
         spatial_sum[numpy.arange(number_of_nodes), self.spatial_mask] = 1
         spatial_sum = spatial_sum.T
-        util.log_debug_array(LOG, spatial_sum, "spatial_sum")
+        self.log.debug("spatial_sum")
+        self.log.debug(narray_describe(spatial_sum))
         nodes_per_area = numpy.sum(spatial_sum, axis=1)[:, numpy.newaxis]
         self.spatial_mean = spatial_sum / nodes_per_area
-        util.log_debug_array(LOG, self.spatial_mean, "spatial_mean", owner=self.__class__.__name__)
+        self.log.debug("spatial_mean")
+        self.log.debug(narray_describe(self.spatial_mean))
 
 
     def sample(self, step, state):
@@ -313,20 +296,18 @@ class SpatialAverage(Monitor):
             monitored_state = numpy.dot(self.spatial_mean, state[self.voi, :])
             return [time, monitored_state.transpose((1, 0, 2))]
 
-    def create_time_series(self, storage_path, connectivity=None, surface=None,
+    def create_time_series(self, connectivity=None, surface=None,
                            region_map=None, region_volume_map=None):
         if self.is_default_special_mask:
-            return TimeSeriesRegion(storage_path=storage_path,
-                                    sample_period=self.period,
+            return TimeSeriesRegion(sample_period=self.period,
                                     region_mapping=region_map,
                                     region_mapping_volume=region_volume_map,
                                     title='Regions ' + self.__class__.__name__,
-                                    connectivity=connectivity,
-                                    **self._transform_user_tags())
+                                    connectivity=connectivity)
         else:
             # mask does not correspond to the number of regions
             # let the parent create a plain TimeSeries
-            return super(SpatialAverage, self).create_time_series(storage_path)
+            return super(SpatialAverage, self).create_time_series()
 
 
 class GlobalAverage(Monitor):
@@ -345,10 +326,10 @@ class GlobalAverage(Monitor):
             data = numpy.mean(state[self.voi, :], axis=1)[:, numpy.newaxis, :]
             return [time, data]
 
-    def create_time_series(self, storage_path, connectivity=None, surface=None,
+    def create_time_series(self, connectivity=None, surface=None,
                            region_map=None, region_volume_map=None):
         # ignore connectivity and surface and let parent create a TimeSeries
-        return super(GlobalAverage, self).create_time_series(storage_path)
+        return super(GlobalAverage, self).create_time_series()
 
 
 class TemporalAverage(Monitor):
@@ -366,7 +347,7 @@ class TemporalAverage(Monitor):
         stock_size = (self.istep, self.voi.shape[0],
                       simulator.number_of_nodes,
                       simulator.model.number_of_modes)
-        LOG.debug("Temporal average stock_size is %s" % (str(stock_size), ))
+        self.log.debug("Temporal average stock_size is %s" % (str(stock_size), ))
         self._stock = numpy.zeros(stock_size)
 
 
@@ -384,17 +365,28 @@ class TemporalAverage(Monitor):
             return [time, avg_stock]
 
 
+# mhtodo: this is not a proper superclass but a mixin, it refers to fields that don't exist
+
 class Projection(Monitor):
     "Base class monitor providing lead field suppport."
     _ui_name = "Projection matrix"
 
-    region_mapping = RegionMapping(
+    region_mapping = Attr(
+        RegionMapping,
         required=False,
-        label="region mapping", order=3,
+        label="region mapping",  #order=3,
         doc="A region mapping specifies how vertices of a surface correspond to given regions in the"
             " connectivity. For iEEG/EEG/MEG monitors, this must be specified when performing a region"
             " simulation but is optional for a surface simulation.")
 
+    obsnoise = Attr(
+        noise.Noise,
+        label="Observation Noise",
+        default=noise.Additive(),
+        required=False,
+        doc="""The monitor's noise source. It incorporates its
+        own instance of Numpy's RandomState.""")
+
     @staticmethod
     def oriented_gain(gain, orient):
         "Apply orientations to gain matrix."
@@ -403,27 +395,22 @@ class Projection(Monitor):
     @classmethod
     def _projection_class(cls):
         if hasattr(cls, 'projection'):
-            return type(cls.projection)
+            return cls.projection.field_type
         else:
             return ProjectionMatrix
 
     @classmethod
     def from_file(cls, sensors_fname, projection_fname, rm_f_name="regionMapping_16k_76.txt",
-                  period=1e3/1024.0, instance=None, **kwds):
+                  period=1e3/1024.0, **kwds):
         """
         Build Projection-based monitor from sensors and projection files, and
         any extra keyword arguments are passed to the monitor class constructor.
 
         """
-        if instance is None:
-            result = cls(**kwds)
-        else:
-            result = instance
-
-        result.sensors = type(cls.sensors).from_file(sensors_fname)
+        result = cls(period=period, **kwds)
+        result.sensors = cls.sensors.field_type.from_file(sensors_fname)
         result.projection = cls._projection_class().from_file(projection_fname)
         result.region_mapping = RegionMapping.from_file(rm_f_name)
-
         return result
 
     def analytic(self, loc, ori):
@@ -443,6 +430,16 @@ class Projection(Monitor):
         if hasattr(self, 'sensors'):
             self.sensors.configure()
 
+        # handle observation noise and configure white/coloured noise
+        # pass in access to the: i) dt and ii) sample shape
+        if self.obsnoise is not None:
+            # configure the noise level
+            if self.obsnoise.ntau > 0.0:
+                noiseshape = self.sensors.labels[:,numpy.newaxis].shape
+                self.obsnoise.configure_coloured(dt=self.dt, shape=noiseshape)
+            else:
+                self.obsnoise.configure_white(dt=self.dt)
+
         # handle region vs simulation, analytic vs numerical proj, cortical vs subcortical.
         # setup convenient locals
         surf = simulator.surface
@@ -462,7 +459,7 @@ class Projection(Monitor):
             else:
                 self.rmap = self.region_mapping
 
-            LOG.debug('Projection used in region sim has %d non-cortical regions', non_cortical_indices.size)
+            self.log.debug('Projection used in region sim has %d non-cortical regions', non_cortical_indices.size)
 
         have_subcortical = len(non_cortical_indices) > 0
 
@@ -473,15 +470,15 @@ class Projection(Monitor):
             sources = {'loc': conn.centres[conn.cortical], 'ori': conn.orientations[conn.cortical]}
 
         # compute analytic if not provided
-        if self.projection is None:
-            LOG.debug('Precomputed projection not unavailable using analytic approximation.')
+        if not hasattr(self, 'projection'):
+            self.log.debug('Precomputed projection not unavailable using analytic approximation.')
             self.gain = self.analytic(**sources)
 
         # reduce to region lead field if region sim
         if not using_cortical_surface and self.gain.shape[1] == self.rmap.size:
             gain = numpy.zeros((self.gain.shape[0], conn.number_of_regions))
             numpy_add_at(gain.T, self.rmap, self.gain.T)
-            LOG.debug('Region mapping gain shape %s to %s', self.gain.shape, gain.shape)
+            self.log.debug('Region mapping gain shape %s to %s', self.gain.shape, gain.shape)
             self.gain = gain
 
         # append analytic sub-cortical to lead field
@@ -489,24 +486,29 @@ class Projection(Monitor):
             # need matrix of shape (proj.shape[0], len(sc_ind))
             src = conn.centres[non_cortical_indices], conn.orientations[non_cortical_indices]
             self.gain = numpy.hstack((self.gain, self.analytic(*src)))
-            LOG.debug('Added subcortical analytic gain, for final shape %s', self.gain.shape)
+            self.log.debug('Added subcortical analytic gain, for final shape %s', self.gain.shape)
 
         if self.sensors.usable is not None and not self.sensors.usable.all():
             mask_unusable = ~self.sensors.usable
             self.gain[mask_unusable] = 0.0
-            LOG.debug('Zeroed gain coefficients for %d unusable sensors', mask_unusable.sum())
+            self.log.debug('Zeroed gain coefficients for %d unusable sensors', mask_unusable.sum())
 
         # unconditionally zero NaN elements; framework not prepared for NaNs.
         nan_mask = numpy.isfinite(self.gain).all(axis=1)
         self.gain[~nan_mask] = 0.0
-        LOG.debug('Zeroed %d NaN gain coefficients', nan_mask.sum())
+        self.log.debug('Zeroed %d NaN gain coefficients', nan_mask.sum())
 
         # attrs used for recording
         self._state = numpy.zeros((self.gain.shape[0], len(self.voi)))
         self._period_in_steps = int(self.period / self.dt)
-        LOG.debug('State shape %s, period in steps %s', self._state.shape, self._period_in_steps)
+        self.log.debug('State shape %s, period in steps %s', self._state.shape, self._period_in_steps)
+
+        self.log.info('Projection configured gain shape %s', self.gain.shape)
+
+
+    def configure(self, *args, **kwargs):
+        self.sensors.configure()
 
-        LOG.info('Projection configured gain shape %s', self.gain.shape)
 
     def sample(self, step, state):
         "Record state, returning sample at sampling frequency / period."
@@ -514,37 +516,43 @@ class Projection(Monitor):
         if step % self._period_in_steps == 0:
             time = (step - self._period_in_steps / 2.0) * self.dt
             sample = self._state.copy() / self._period_in_steps
+
+            # add observation noise if available
+            if self.obsnoise is not None:
+                sample += self.obsnoise.generate(shape=sample.shape)
+
             self._state[:] = 0.0
             return time, sample.T[..., numpy.newaxis] # for compatibility
 
     _gain = None
 
-    def _get_gain(self):
+    @property
+    def gain(self):
         if self._gain is None:
             self._gain = self.projection.projection_data
         return self._gain
 
-    def _set_gain(self, new_gain):
+    @gain.setter
+    def gain(self, new_gain):
         self._gain = new_gain
 
-    gain = property(_get_gain, _set_gain)
-
     _rmap = None
 
     def _reg_map_data(self, reg_map):
         return getattr(reg_map, 'array_data', reg_map)
 
-    def _get_rmap(self):
+    @property
+    def rmap(self):
         "Normalize obtaining reg map vector over various possibilities."
         if self._rmap is None:
             self._rmap = self._reg_map_data(self.region_mapping)
         return self._rmap
 
-    def _set_rmap(self, reg_map):
+    @rmap.setter
+    def rmap(self, reg_map):
         if self._rmap is not None:
             self._rmap = self._reg_map_data(self.region_mapping)
 
-    rmap = property(_get_rmap, _set_rmap)
 
 
 class EEG(Projection):
@@ -562,27 +570,32 @@ class EEG(Projection):
     """
     _ui_name = "EEG"
 
-    projection = ProjectionSurfaceEEG(
-        default=None, label='Projection matrix', order=2,
+    projection = Attr(
+        ProjectionSurfaceEEG,
+        default=None, label='Projection matrix',  #order=2,
         doc='Projection matrix to apply to sources.')
 
-    reference = basic.String(required=False, label="EEG Reference", order=5,
-                             doc='EEG Electrode to be used as reference, or "average" to '
-                                 'apply an average reference. If none is provided, the '
-                                 'produced time-series are the idealized or reference-free.')
+    reference = Attr(
+        str, required=False,
+        label="EEG Reference",  #order=5,
+        doc='EEG Electrode to be used as reference, or "average" to '
+            'apply an average reference. If none is provided, the '
+            'produced time-series are the idealized or reference-free.')
 
-    sensors = SensorsEEG(required=True, label="EEG Sensors", order=1,
-                         doc='Sensors to use for this EEG monitor')
+    sensors = Attr(SensorsEEG, required=True, label="EEG Sensors",  #order=1,
+                   doc='Sensors to use for this EEG monitor')
 
-    sigma = basic.Float(label="Conductivity (w/o projection)", default=1.0, order=4,
-                        doc='When a projection matrix is not used, this provides '
-                            'the value of conductivity in the formula for the single '
-                            'sphere approximation of the head (Sarvas 1987).')
+    sigma = Float(
+        default=1.0,  #order=4,
+        label="Conductivity (w/o projection)",
+        doc='When a projection matrix is not used, this provides '
+            'the value of conductivity in the formula for the single '
+            'sphere approximation of the head (Sarvas 1987).')
 
 
     @classmethod
     def from_file(cls, sensors_fname='eeg_brainstorm_65.txt', projection_fname='projection_eeg_65_surface_16k.npy', **kwargs):
-        return Projection.from_file.im_func(cls, sensors_fname, projection_fname, **kwargs)
+        return Projection.from_file.__func__(cls, sensors_fname, projection_fname, **kwargs)
 
     def config_for_sim(self, simulator):
         super(EEG, self).config_for_sim(simulator)
@@ -622,35 +635,34 @@ class EEG(Projection):
             sample -= self._ref_vec.dot(sample[:, self._ref_vec_mask])[:, numpy.newaxis]
             return time, sample.reshape((state.shape[0], -1, 1))
 
-    def create_time_series(self, storage_path, connectivity=None, surface=None,
+    def create_time_series(self, connectivity=None, surface=None,
                            region_map=None, region_volume_map=None):
-        return TimeSeriesEEG(storage_path=storage_path,
-                             sensors=self.sensors,
+        return TimeSeriesEEG(sensors=self.sensors,
                              sample_period=self.period,
-                             title=' ' + self.__class__.__name__,
-                             **self._transform_user_tags())
+                             title=' ' + self.__class__.__name__)
 
 
 class MEG(Projection):
     "Forward solution monitor for magnetoencephalography (MEG)."
     _ui_name = "MEG"
 
-    projection = ProjectionSurfaceMEG(
-        default=None, label='Projection matrix', order=2,
+    projection = Attr(
+        ProjectionSurfaceMEG,
+        default=None, label='Projection matrix', # order=2,
         doc='Projection matrix to apply to sources.')
 
-    sensors = sensors_module.SensorsMEG(
+    sensors = Attr(
+        sensors_module.SensorsMEG,
         label = "MEG Sensors",
         default = None,
-        required = True, order=1,
-        doc = """The set of MEG sensors for which the forward solution will be
-        calculated.""")
+        required = True,  #order=1,
+        doc="The set of MEG sensors for which the forward solution will be calculated.")
 
 
     @classmethod
     def from_file(cls, sensors_fname='meg_brainstorm_276.txt',
                    projection_fname='projection_meg_276_surface_16k.npy', **kwargs):
-        return Projection.from_file.im_func(cls, sensors_fname, projection_fname, **kwargs)
+        return Projection.from_file.__func__(cls, sensors_fname, projection_fname, **kwargs)
 
     def analytic(self, loc, ori):
         """Compute single sphere analytic form of MEG lead field.
@@ -686,13 +698,11 @@ class MEG(Projection):
                                                                      (rsk * delF), axis=1)[:, numpy.newaxis]))
         return numpy.sqrt(numpy.sum(B_r**2, axis=2))
 
-    def create_time_series(self, storage_path, connectivity=None, surface=None,
+    def create_time_series(self, connectivity=None, surface=None,
                            region_map=None, region_volume_map=None):
-        return TimeSeriesMEG(storage_path=storage_path,
-                             sensors=self.sensors,
+        return TimeSeriesMEG(sensors=self.sensors,
                              sample_period=self.period,
-                             title=' ' + self.__class__.__name__,
-                             **self._transform_user_tags())
+                             title=' ' + self.__class__.__name__)
 
 
 class iEEG(Projection):
@@ -700,21 +710,23 @@ class iEEG(Projection):
 
     _ui_name = "Intracerebral / Stereo EEG"
 
-    projection = ProjectionSurfaceSEEG(
-        default=None, label='Projection matrix', order=2,
+    projection = Attr(
+        ProjectionSurfaceSEEG,
+        default=None, label='Projection matrix',  #order=2,
         doc='Projection matrix to apply to sources.')
 
-    sigma = basic.Float(label="conductivity", default=1.0, order=4)
+    sigma = Float(label="conductivity", default=1.0)  #, order=4)
 
-    sensors = sensors_module.SensorsInternal(
-        label="Internal brain sensors", default=None, required=True, order=1,
+    sensors = Attr(
+        sensors_module.SensorsInternal,
+        label="Internal brain sensors", default=None, required=True,  #order=1,
         doc="The set of SEEG sensors for which the forward solution will be calculated.")
 
 
     @classmethod
     def from_file(cls, sensors_fname='seeg_588.txt',
                    projection_fname='projection_seeg_588_surface_16k.npy', **kwargs):
-        return Projection.from_file.im_func(cls, sensors_fname, projection_fname, **kwargs)
+        return Projection.from_file.__func__(cls, sensors_fname, projection_fname, **kwargs)
 
     def analytic(self, loc, ori):
         """Compute the projection matrix -- simple distance weight for now.
@@ -729,13 +741,11 @@ class iEEG(Projection):
             V_r[sensor_k, :] = numpy.sum(Q * (a / na ** 3), axis=1) / (4.0 * numpy.pi * self.sigma)
         return V_r
 
-    def create_time_series(self, storage_path, connectivity=None, surface=None,
+    def create_time_series(self, connectivity=None, surface=None,
                            region_map=None, region_volume_map=None):
-        return TimeSeriesSEEG(storage_path=storage_path,
-                              sensors=self.sensors,
+        return TimeSeriesSEEG(sensors=self.sensors,
                               sample_period=self.period,
-                              title=' ' + self.__class__.__name__,
-                              **self._transform_user_tags())
+                              title=' ' + self.__class__.__name__)
 
 
 class Bold(Monitor):
@@ -781,25 +791,26 @@ class Bold(Monitor):
     """
     _ui_name = "BOLD"
 
-    period = basic.Float(
-        label = "Sampling period (ms)",
-        default = 2000.0,
-        doc = """For the BOLD monitor, sampling period in milliseconds must be
+    period = Float(
+        label="Sampling period (ms)",
+        default=2000.0,
+        doc="""For the BOLD monitor, sampling period in milliseconds must be
         an integral multiple of 500. Typical measurment interval (repetition
         time TR) is between 1-3 s. If TR is 2s, then Bold period is 2000ms.""")
 
-    hrf_kernel = equations.HRFKernelEquation(
-        label = "Haemodynamic Response Function",
-        default = equations.FirstOrderVolterra,
-        required = True,
-        doc = """A tvb.datatypes.equation object which describe the haemodynamic
+    hrf_kernel = Attr(
+        equations.HRFKernelEquation,
+        label="Haemodynamic Response Function",
+        default=equations.FirstOrderVolterra(),
+        required=True,
+        doc="""A tvb.datatypes.equation object which describe the haemodynamic
         response function used to compute the BOLD signal.""")
 
-    hrf_length = basic.Float(
-        label = "Duration (ms)",
-        default = 20000.,
-        doc= """Duration of the hrf kernel""",
-        order=-1)
+    hrf_length = Float(
+        label="Duration (ms)",
+        default=20000.,
+        doc= """Duration of the hrf kernel""",)
+        #order=-1)
 
     _interim_period = None
     _interim_istep = None
@@ -822,17 +833,16 @@ class Bold(Monitor):
         stock_time_max    = magic_number/1000.0                                # [s]
         stock_time_step   = stock_time_max / self._stock_steps                 # [s]
         self._stock_time  = numpy.arange(0.0, stock_time_max, stock_time_step) # [s]
-        LOG.debug("Bold requires %d steps for HRF kernel convolution", self._stock_steps)
+        self.log.debug("Bold requires %d steps for HRF kernel convolution", self._stock_steps)
         #Compute the HRF kernel
-        self.hrf_kernel.pattern = self._stock_time
-        G = self.hrf_kernel.pattern
+        G = self.hrf_kernel.evaluate(self._stock_time)
         #Reverse it, need it into the past for matrix-multiply of stock
         G = G[::-1]
         self.hemodynamic_response_function = G[numpy.newaxis, :]
         #Interim stock configuration
         self._interim_period = 1.0 / self._stock_sample_rate #period in ms
         self._interim_istep = int(round(self._interim_period / self.dt)) # interim period in integration time steps
-        LOG.debug('Bold HRF shape %s, interim period & istep %d & %d',
+        self.log.debug('Bold HRF shape %s, interim period & istep %d & %d',
                   self.hemodynamic_response_function.shape, self._interim_period, self._interim_istep)
 
     def config_for_sim(self, simulator):
@@ -840,10 +850,10 @@ class Bold(Monitor):
         self.compute_hrf()
         sample_shape = self.voi.shape[0], simulator.number_of_nodes, simulator.model.number_of_modes
         self._interim_stock = numpy.zeros((self._interim_istep,) + sample_shape)
-        LOG.debug("BOLD inner buffer %s %.2f MB" % (
+        self.log.debug("BOLD inner buffer %s %.2f MB" % (
             self._interim_stock.shape, self._interim_stock.nbytes/2**20))
         self._stock = numpy.zeros((self._stock_steps,) + sample_shape)
-        LOG.debug("BOLD outer buffer %s %.2f MB" % (
+        self.log.debug("BOLD outer buffer %s %.2f MB" % (
             self._stock.shape, self._stock.nbytes/2**20))
 
 
@@ -853,13 +863,13 @@ class Bold(Monitor):
         # At stock's period update it with the temporal average of interim-stock
         if step % self._interim_istep == 0:
             avg_interim_stock = numpy.mean(self._interim_stock, axis=0)
-            self._stock[((step/self._interim_istep % self._stock_steps) - 1), :] = avg_interim_stock
+            self._stock[((step//self._interim_istep % self._stock_steps) - 1), :] = avg_interim_stock
         # At the monitor's period, apply the heamodynamic response function to
         # the stock and return the resulting BOLD signal.
         if step % self.istep == 0:
             time = step * self.dt
             hrf = numpy.roll(self.hemodynamic_response_function,
-                             ((step/self._interim_istep % self._stock_steps) - 1),
+                             ((step//self._interim_istep % self._stock_steps) - 1),
                              axis=1)
             if isinstance(self.hrf_kernel, equations.FirstOrderVolterra):
                 k1_V0 = self.hrf_kernel.parameters["k_1"] * self.hrf_kernel.parameters["V_0"]
@@ -903,7 +913,6 @@ class ProgressLogger(Monitor):
 
     def __init__(self, **kwargs):
         super(ProgressLogger, self).__init__(**kwargs)
-        self.logger = get_logger('Sim Progress')
 
     def config_for_sim(self, simulator):
         self._dt = simulator.integrator.dt
@@ -915,4 +924,7 @@ class ProgressLogger(Monitor):
         except:
             self._last_step = step
         if (step - self._last_step) % self._istep == 0:
-            self.logger.info('step %d time %.4f s', step, step * self._dt / 1e3)
+            self.log.info('step %d time %.4f s', step, step * self._dt / 1e3)
+
+    def sample(self, step, state):
+        raise NotImplementedError
\ No newline at end of file
