@@ -37,29 +37,25 @@ A collection of plotting functions used by simulator/demos
 
 import numpy
 import scipy as sp
-import networkx as nx
+import matplotlib.pyplot as pyplot
+import matplotlib.colors
+import matplotlib.ticker as ticker
 from tvb.basic.logger.builder import get_logger
 
 LOG = get_logger(__name__)
 
-
-##----------------------------------------------------------------------------##
-##-                  matplotlib based plotting functions                     -##
-##---------------------------------------------------------------------------cd-##
-
-import matplotlib as mpl
-import matplotlib.pyplot as pyplot
-import matplotlib.colors
-import matplotlib.ticker as ticker
-import matplotlib.colors as colors
-
-
 try:
-    from mpl_toolkits.axes_grid import make_axes_locatable
+    from mpl_toolkits.axes_grid1 import make_axes_locatable
+    from mpl_toolkits.mplot3d import Axes3D
+
     IMPORTED_MPL_TOOLKITS = True
+
 except ImportError:
     IMPORTED_MPL_TOOLKITS = False
-    LOG.error("You need mpl_toolkits")
+    make_axes_locatable = None
+    Axes3D = None
+    LOG.warn("You need mpl_toolkits for some axe related features")
+
 
 def _blob(x, y, area, colour):
     """
@@ -73,8 +69,7 @@ def _blob(x, y, area, colour):
     pyplot.fill(xcorners, ycorners, colour, edgecolor=colour)
 
 
-
-def hinton_diagram(connectivity_weights, num, maxWeight=None):
+def hinton_diagram(connectivity_weights, num, max_weight=None):
     """
     Draws a Hinton diagram. This function temporarily disables matplotlib
     interactive mode if it is on, otherwise this takes forever.
@@ -82,10 +77,10 @@ def hinton_diagram(connectivity_weights, num, maxWeight=None):
     weights_figure = pyplot.figure(num=num)
     height, width = connectivity_weights.shape
 
-    if not maxWeight:
-        maxWeight = 2 ** numpy.ceil(numpy.log(numpy.max(numpy.abs(connectivity_weights))) / numpy.log(2))
+    if not max_weight:
+        max_weight = 2 ** numpy.ceil(numpy.log(numpy.max(numpy.abs(connectivity_weights))) / numpy.log(2))
 
-    #pyplot.fill(numpy.array([0,width,width,0]),numpy.array([0,0,height+0.5,height+0.5]),'gray')
+    # pyplot.fill(numpy.array([0,width,width,0]),numpy.array([0,0,height+0.5,height+0.5]),'gray')
     pyplot.axis('equal')
     weights_axes = weights_figure.gca()
 
@@ -95,13 +90,12 @@ def hinton_diagram(connectivity_weights, num, maxWeight=None):
             _y = y + 1
             w = connectivity_weights[y, x]
             if w > 0:
-                _blob(_x - 1., height - _y + 0.0, min(1, w / maxWeight), 'red')
+                _blob(_x - 1., height - _y + 0.0, min(1, w / max_weight), 'red')
             elif w < 0:
-                _blob(_x - 1., height - _y + 0.0, min(1, -w / maxWeight), 'black')
+                _blob(_x - 1., height - _y + 0.0, min(1, -w / max_weight), 'black')
     return weights_axes
 
 
-
 def plot_connectivity(connectivity, num="weights", order_by=None, plot_hinton=False, plot_tracts=True):
     """
     A 2D plot for visualizing the Connectivity.weights matrix
@@ -157,7 +151,6 @@ def plot_connectivity(connectivity, num="weights", order_by=None, plot_hinton=Fa
         tracts_axes.set_xticklabels(list(labels[order]), fontsize=8, rotation=90)
 
 
-
 def plot_local_connectivity(cortex, cutoff=None):
     """
     Display the local connectivity function as a line plot. Four lines are
@@ -177,7 +170,7 @@ def plot_local_connectivity(cortex, cutoff=None):
     Usage, from demos directory, with tvb in your path ::
         
         import tvb.datatypes.surfaces as surfaces
-        import plotting_tools
+        import tvb.simulator.plot.tools as plotting_tools
         cortex = surfaces.Cortex()
         plotting_tools.plot_local_connectivity(cortex, cutoff=60.)
         plotting_tools.pyplot.show()
@@ -186,11 +179,10 @@ def plot_local_connectivity(cortex, cutoff=None):
 
     dashes = ['--',  # : dashed line   -- blue
               '-.',  # : dash-dot line -- red
-              ':',   # : dotted line   -- green
-              '-']   # : solid line    -- black
+              ':',  # : dotted line   -- green
+              '-']  # : solid line    -- black
 
-
-    #If necessary, add a default LocalConnectivity to ``local_connectivity``.
+    # If necessary, add a default LocalConnectivity to ``local_connectivity``.
     if cortex.local_connectivity is None:
         LOG.info("local_connectivity is None, adding default LocalConnectivity")
         cortex.local_connectivity = cortex.trait["local_connectivity"]
@@ -198,7 +190,7 @@ def plot_local_connectivity(cortex, cutoff=None):
     if cutoff:
         cortex.local_connectivity.cutoff = cutoff
 
-    #We need a cutoff distance to work from...
+    # We need a cutoff distance to work from...
     if cortex.local_connectivity.cutoff is None:
         LOG.error("You need to provide a cutoff...")
         return
@@ -211,39 +203,35 @@ def plot_local_connectivity(cortex, cutoff=None):
 
     # ideally all these lines should overlap
 
-    #What we want
+    # What we want
     hi_res = 1024
     step = 2.0 * cutoff_2 / (hi_res - 1)
     hi_x = numpy.arange(-cutoff_2, cutoff_2 + step, step)
-    cortex.local_connectivity.equation.pattern = numpy.abs(hi_x)
-    pyplot.plot(hi_x, cortex.local_connectivity.equation.pattern, 'k',
+    pyplot.plot(hi_x, cortex.local_connectivity.equation.evaluate(numpy.abs(hi_x)), 'k',
                 linestyle=dashes[-1], linewidth=3)
 
-    #What we'll mostly get
+    # What we'll mostly get
     avg_res = 2 * int(cutoff / cortex.edge_length_mean)
     step = cutoff_2 / (avg_res - 1)
     avg_x = numpy.arange(-cutoff, cutoff + step, step)
-    cortex.local_connectivity.equation.pattern = numpy.abs(avg_x)
-    pyplot.plot(avg_x, cortex.local_connectivity.equation.pattern, 'b',
+    pyplot.plot(avg_x, cortex.local_connectivity.equation.evaluate(numpy.abs(avg_x)), 'b',
                 linestyle=dashes[0], linewidth=3)
 
-    #It can be this bad
+    # It can be this bad
     worst_res = 2 * int(cutoff / cortex.edge_length_max)
     step = cutoff_2 / (worst_res - 1)
     worst_x = numpy.arange(-cutoff, cutoff + step, step)
-    cortex.local_connectivity.equation.pattern = numpy.abs(worst_x)
-    pyplot.plot(worst_x, cortex.local_connectivity.equation.pattern, 'r',
+    pyplot.plot(worst_x, cortex.local_connectivity.equation.evaluate(numpy.abs(worst_x)), 'r',
                 linestyle=dashes[1], linewidth=3)
 
-    #This is as good as it gets...
+    # This is as good as it gets...
     best_res = 2 * int(cutoff / cortex.edge_length_min)
     step = cutoff_2 / (best_res - 1)
     best_x = numpy.arange(-cutoff, cutoff + step, step)
-    cortex.local_connectivity.equation.pattern = numpy.abs(best_x)
-    pyplot.plot(best_x, cortex.local_connectivity.equation.pattern, 'g',
+    pyplot.plot(best_x, cortex.local_connectivity.equation.evaluate(numpy.abs(best_x)), 'g',
                 linestyle=dashes[2], linewidth=3)
 
-    #Plot the cutoff
+    # Plot the cutoff
     ymin, ymax = pyplot.ylim()
     pyplot.plot([-cutoff, -cutoff], [ymin, ymax], "k--")
     pyplot.plot([cutoff, cutoff], [ymin, ymax], "k--")
@@ -254,11 +242,10 @@ def plot_local_connectivity(cortex, cutoff=None):
     pyplot.legend(("Theoretical", "Typical", "Worst", "Best", "Cutoff"))
 
     # set the linewidth of the first legend object
-    #leg.legendHandles[0].set_linewidth(6.0)
-    #leg.legendHandles[1].set_linewidth(6.0)
-    #leg.legendHandles[2].set_linewidth(6.0)
-    #leg.legendHandles[3].set_linewidth(6.0)
-
+    # leg.legendHandles[0].set_linewidth(6.0)
+    # leg.legendHandles[1].set_linewidth(6.0)
+    # leg.legendHandles[2].set_linewidth(6.0)
+    # leg.legendHandles[3].set_linewidth(6.0)
 
 
 def plot_pattern(pattern_object):
@@ -269,7 +256,7 @@ def plot_pattern(pattern_object):
     pyplot.subplot(221)
     pyplot.plot(pattern_object.spatial_pattern, "k*")
     pyplot.title("Space")
-    #pyplot.plot(pattern_object.space, pattern_object.spatial_pattern, "k*")
+    # pyplot.plot(pattern_object.space, pattern_object.spatial_pattern, "k*")
     pyplot.subplot(223)
     pyplot.plot(pattern_object.time.T, pattern_object.temporal_pattern.T)
     pyplot.title("Time")
@@ -279,8 +266,7 @@ def plot_pattern(pattern_object):
     pyplot.title("Stimulus")
     pyplot.xlabel("Time")
     pyplot.ylabel("Space")
-    #pyplot.show()
-
+    # pyplot.show()
 
 
 def show_me_the_colours():
@@ -300,24 +286,20 @@ def show_me_the_colours():
         ax.text(0.05, 0.5, colours[k])
 
 
-
 def plot_matrix(mat, fig_name='plot_this_matrix', connectivity=None, binary_matrix=False):
     """
     An embellished matshow display
     """
-    #NOTE: I could add more stuff in plot_connectivity, but I rather have
+    # NOTE: I could add more stuff in plot_connectivity, but I rather have
     # a dummy function for displaying a pretty matrix with the 
     # value of each element.
 
-    from matplotlib import colors
-
-    fig, ax = pyplot.subplots(num=fig_name, figsize=(12,10))
-
+    fig, ax = pyplot.subplots(num=fig_name, figsize=(12, 10))
 
     if binary_matrix:
-        cmap = colors.ListedColormap(['black', 'white'])
-        bounds=[0,1,2]
-        norm = colors.BoundaryNorm(bounds, cmap.N)
+        cmap = matplotlib.colors.ListedColormap(['black', 'white'])
+        bounds = [0, 1, 2]
+        norm = matplotlib.colors.BoundaryNorm(bounds, cmap.N)
 
         p = ax.pcolormesh(mat, cmap=cmap, norm=norm, edgecolors='k')
         ax.invert_yaxis()
@@ -333,46 +315,31 @@ def plot_matrix(mat, fig_name='plot_this_matrix', connectivity=None, binary_matr
     if connectivity is not None:
         order = numpy.arange(connectivity.number_of_regions)
         labels = connectivity.region_labels
-        pyplot.xticks(numpy.arange(connectivity.number_of_regions)+0.5, list(labels[order]), fontsize=10, rotation=90)
-        pyplot.yticks(numpy.arange(connectivity.number_of_regions)+0.5, list(labels[order]), fontsize=10)
-    
-    width  = mat.shape[0]
-    height = mat.shape[1]
-
-    # for x in range(width):
-    #     for y in range(height):
-    #         ax.annotate(str(int(mat[x][y])),
-    #                     xy=(y, x),
-    #                     horizontalalignment='center',
-    #                     verticalalignment  = 'center',
-    #                     fontsize=10)
-
+        pyplot.xticks(numpy.arange(connectivity.number_of_regions) + 0.5, list(labels[order]), fontsize=10, rotation=90)
+        pyplot.yticks(numpy.arange(connectivity.number_of_regions) + 0.5, list(labels[order]), fontsize=10)
 
 
 def plot_3d_centres(xyz):
+    if not IMPORTED_MPL_TOOLKITS:
+        LOG.warn("3D axes will be enabled only if you install ")
+        return
 
-        import matplotlib as mpl
-        from mpl_toolkits.mplot3d import Axes3D
-        import matplotlib.pyplot as plt
-
-
-        fig = plt.figure(1)
-        fig.clf()
-        ax = Axes3D(fig)
-        ax.plot(xyz[:, 0], xyz[:, 1], xyz[:, 2], 'o', alpha=0.6)
-        ax.set_xlim([min(xyz[:, 0]), max(xyz[:, 0])])
-        ax.set_ylim([min(xyz[:, 1]), max(xyz[:, 1])])
-        ax.set_zlim([min(xyz[:, 2]), max(xyz[:, 2])])
-        ax.set_xlabel('x [mm]')
-        ax.set_ylabel('y [mm]')
-        ax.set_zlabel('z [mm]')
-
+    fig = pyplot.figure(1)
+    fig.clf()
+    ax = Axes3D(fig)
+    ax.plot(xyz[:, 0], xyz[:, 1], xyz[:, 2], 'o', alpha=0.6)
+    ax.set_xlim([min(xyz[:, 0]), max(xyz[:, 0])])
+    ax.set_ylim([min(xyz[:, 1]), max(xyz[:, 1])])
+    ax.set_zlim([min(xyz[:, 2]), max(xyz[:, 2])])
+    ax.set_xlabel('x [mm]')
+    ax.set_ylabel('y [mm]')
+    ax.set_zlabel('z [mm]')
 
 
 def plot_tri_matrix(mat, figure=None, num='plot_part_of_this_matrix', size=None,
-                        cmap=pyplot.cm.RdBu_r, colourbar=True,
-                        color_anchor=None, node_labels=None, x_tick_rot=0, 
-                        title=None):
+                    cmap=pyplot.cm.RdBu_r, colourbar=True,
+                    color_anchor=None, node_labels=None, x_tick_rot=0,
+                    title=None):
     r"""Creates a lower-triangle of a square matrix. Very often found to display correlations or coherence.
 
     Parameters
@@ -407,7 +374,7 @@ def plot_tri_matrix(mat, figure=None, num='plot_part_of_this_matrix', size=None,
 
     if figure is not None:
         fig = figure
-    else :
+    else:
         if num is None:
             fig = pyplot.figure()
         else:
@@ -422,25 +389,24 @@ def plot_tri_matrix(mat, figure=None, num='plot_part_of_this_matrix', size=None,
 
     ax_im = fig.add_subplot(1, 1, 1)
 
-    N   = mat.shape[0]
-    idx = numpy.arange(N) 
+    N = mat.shape[0]
+    idx = numpy.arange(N)
 
-     
     if colourbar:
         if IMPORTED_MPL_TOOLKITS:
             divider = make_axes_locatable(ax_im)
-            ax_cb   = divider.new_vertical(size="10%", pad=0.1, pack_start=True)
+            ax_cb = divider.new_vertical(size="10%", pad=0.1, pack_start=True)
             fig.add_axes(ax_cb)
         else:
             pass
 
     mat_copy = mat.copy()
 
-    #Null the upper triangle, including the main diagonal.
-    idx_null           = numpy.triu_indices(mat_copy.shape[0])
+    # Null the upper triangle, including the main diagonal.
+    idx_null = numpy.triu_indices(mat_copy.shape[0])
     mat_copy[idx_null] = numpy.nan
 
-    #Min max values
+    # Min max values
     max_val = numpy.nanmax(mat_copy)
     min_val = numpy.nanmin(mat_copy)
 
@@ -450,19 +416,19 @@ def plot_tri_matrix(mat, figure=None, num='plot_part_of_this_matrix', size=None,
     elif color_anchor == 0:
         bound = max(abs(max_val), abs(min_val))
         color_min = -bound
-        color_max =  bound
+        color_max = bound
     else:
         color_min = color_anchor[0]
         color_max = color_anchor[1]
 
-    #The call to imshow produces the matrix plot:
+    # The call to imshow produces the matrix plot:
     im = ax_im.imshow(mat_copy, origin='upper', interpolation='nearest',
                       vmin=color_min, vmax=color_max, cmap=cmap)
 
-    #Formatting:
+    # Formatting:
     ax = ax_im
     ax.grid(True)
-    #Label each of the cells with the row and the column:
+    # Label each of the cells with the row and the column:
     if node_labels is not None:
         for i in range(0, mat_copy.shape[0]):
             if i < (mat_copy.shape[0] - 1):
@@ -480,7 +446,7 @@ def plot_tri_matrix(mat, figure=None, num='plot_part_of_this_matrix', size=None,
         ax.set_ybound([-0.5, N - 0.5])
         ax.set_xbound([-0.5, N - 1.5])
 
-    #Make the tick-marks invisible:
+    # Make the tick-marks invisible:
     for line in ax.xaxis.get_ticklines():
         line.set_markeredgewidth(0)
 
@@ -493,16 +459,15 @@ def plot_tri_matrix(mat, figure=None, num='plot_part_of_this_matrix', size=None,
         ax.set_title(title)
 
     if colourbar:
-        #Set the ticks - if 0 is in the interval of values, set that, as well
-        #as the min, max values:
+        # Set the ticks - if 0 is in the interval of values, set that, as well
+        # as the min, max values:
         if min_val < 0:
             ticks = [color_min, min_val, 0, max_val, color_max]
-        #set the min, mid and  max values:
+        # set the min, mid and  max values:
         else:
-            ticks = [color_min, min_val, (color_max- color_min)/2., max_val, color_max]
+            ticks = [color_min, min_val, (color_max - color_min) / 2., max_val, color_max]
 
-
-        #colourbar:
+        # colourbar:
         if IMPORTED_MPL_TOOLKITS:
             cb = fig.colorbar(im, cax=ax_cb, orientation='horizontal',
                               cmap=cmap,
@@ -525,8 +490,8 @@ def plot_tri_matrix(mat, figure=None, num='plot_part_of_this_matrix', size=None,
     return fig
 
 
-def plot_fast_kde(x, y, kern_nx = None, kern_ny = None, gridsize=(500, 500), 
-             extents=None, nocorrelation=False, weights=None, norm = True, pdf=False, **kwargs):
+def plot_fast_kde(x, y, kern_nx=None, kern_ny=None, gridsize=(500, 500),
+                  extents=None, nocorrelation=False, weights=None, norm=True, pdf=False, **kwargs):
     """
     A faster gaussian kernel density estimate (KDE).  Intended for
     computing the KDE on a regular grid (different use case than
@@ -577,11 +542,11 @@ def plot_fast_kde(x, y, kern_nx = None, kern_ny = None, gridsize=(500, 500),
     **Output**:
         A gridded 2D kernel density estimate of the input points. 
     """
-   
-    #---- Setup --------------------------------------------------------------
+
+    # ---- Setup --------------------------------------------------------------
     x, y = numpy.asarray(x), numpy.asarray(y)
     x, y = numpy.squeeze(x), numpy.squeeze(y)
-    
+
     if x.size != y.size:
         raise ValueError('Input x & y arrays must be the same size!')
 
@@ -595,23 +560,23 @@ def plot_fast_kde(x, y, kern_nx = None, kern_ny = None, gridsize=(500, 500),
         weights = numpy.squeeze(numpy.asarray(weights))
         if weights.size != x.size:
             raise ValueError('Input weights must be an array of the same size'
-                    ' as input x & y arrays!')
+                             ' as input x & y arrays!')
 
     # Default extents are the extent of the data
     if extents is None:
         xmin, xmax = x.min(), x.max()
         ymin, ymax = y.min(), y.max()
     else:
-        xmin, xmax, ymin, ymax = map(float, extents)
-        
+        xmin, xmax, ymin, ymax = list(map(float, extents))
+
     dx = (xmax - xmin) / (nx - 1)
     dy = (ymax - ymin) / (ny - 1)
 
-    #---- Preliminary Calculations -------------------------------------------
+    # ---- Preliminary Calculations -------------------------------------------
 
     # First convert x & y over to pixel coordinates
     # (Avoiding np.digitize due to excessive memory usage!)
-    xyi = numpy.vstack((x,y)).T
+    xyi = numpy.vstack((x, y)).T
     xyi -= [xmin, ymin]
     xyi /= [dx, dy]
     xyi = numpy.floor(xyi, xyi).T
@@ -624,26 +589,26 @@ def plot_fast_kde(x, y, kern_nx = None, kern_ny = None, gridsize=(500, 500),
     cov = numpy.cov(xyi)
 
     if nocorrelation:
-        cov[1,0] = 0
-        cov[0,1] = 0
+        cov[1, 0] = 0
+        cov[0, 1] = 0
 
     # Scaling factor for bandwidth
-    scotts_factor = numpy.power(n, -1.0 / 6) # For 2D
+    scotts_factor = numpy.power(n, -1.0 / 6)  # For 2D
 
-    #---- Make the gaussian kernel -------------------------------------------
+    # ---- Make the gaussian kernel -------------------------------------------
 
     # First, determine how big the kernel needs to be
     std_devs = numpy.diag(numpy.sqrt(cov))
 
-    if kern_nx is None or kern_ny is None: 
+    if kern_nx is None or kern_ny is None:
         kern_nx, kern_ny = numpy.round(scotts_factor * 2 * numpy.pi * std_devs)
-    
-    else: 
+
+    else:
         kern_nx = numpy.round(kern_nx / dx)
         kern_ny = numpy.round(kern_ny / dy)
 
     # Determine the bandwidth to use for the gaussian kernel
-    inv_cov = numpy.linalg.inv(cov * scotts_factor**2) 
+    inv_cov = numpy.linalg.inv(cov * scotts_factor ** 2)
 
     # x & y (pixel) coords of the kernel grid, with <x,y> = <0,0> in center
     xx = numpy.arange(kern_nx, dtype=numpy.float) - kern_nx / 2.0
@@ -652,12 +617,12 @@ def plot_fast_kde(x, y, kern_nx = None, kern_ny = None, gridsize=(500, 500),
 
     # Then evaluate the gaussian function on the kernel grid
     kernel = numpy.vstack((xx.flatten(), yy.flatten()))
-    kernel = numpy.dot(inv_cov, kernel) * kernel 
-    kernel = numpy.sum(kernel, axis=0) / 2.0 
-    kernel = numpy.exp(-kernel) 
+    kernel = numpy.dot(inv_cov, kernel) * kernel
+    kernel = numpy.sum(kernel, axis=0) / 2.0
+    kernel = numpy.exp(-kernel)
     kernel = kernel.reshape((kern_ny, kern_nx))
 
-    #---- Produce the kernel density estimate --------------------------------
+    # ---- Produce the kernel density estimate --------------------------------
 
     # Convolve the gaussian kernel with the 2D histogram, producing a gaussian
     # kernel density estimate on a regular grid
@@ -665,14 +630,14 @@ def plot_fast_kde(x, y, kern_nx = None, kern_ny = None, gridsize=(500, 500),
 
     # Normalization factor to divide result by so that units are in the same
     # units as scipy.stats.kde.gaussian_kde's output.  
-    norm_factor = 2 * numpy.pi * cov * scotts_factor**2
+    norm_factor = 2 * numpy.pi * cov * scotts_factor ** 2
     norm_factor = numpy.linalg.det(norm_factor)
-    #norm_factor = n * dx * dy * np.sqrt(norm_factor)
+    # norm_factor = n * dx * dy * np.sqrt(norm_factor)
     norm_factor = numpy.sqrt(norm_factor)
-    
-    if norm : 
+
+    if norm:
         norm_factor *= n * dx * dy
-    #---- Produce pdf                        --------------------------------
+    # ---- Produce pdf                        --------------------------------
 
     if pdf:
         norm_factor, _ = sp.integrate.nquad(grid, [[xmin, xmax], [ymin, ymax]])
@@ -682,9 +647,3 @@ def plot_fast_kde(x, y, kern_nx = None, kern_ny = None, gridsize=(500, 500),
 
     return grid
 
-if __name__ == '__main__':
-    # Do some stuff that tests or makes use of this module... 
-    pass
-
-
-##- EoF -##
